@@ -1,5 +1,5 @@
-/* Copyright (C) 1995, 1996, 1997, 1998, 2000, 2001, 2004, 2006, 2010,
- *   2012 Free Software Foundation, Inc.
+/* Copyright (C) 1995-1998, 2000, 2001, 2004, 2006, 2010, 2012-2014
+ *   Free Software Foundation, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -40,9 +40,7 @@
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
-#ifdef HAVE_UNISTD_H
 #include <unistd.h>
-#endif
 
 /* For Windows... */
 #ifdef HAVE_IO_H
@@ -80,7 +78,8 @@ SCM_DEFINE (scm_error_scm, "scm-error", 5, 0, 0,
 	    "@code{system-error} then it should be a list containing the\n"
 	    "Unix @code{errno} value; If @var{key} is @code{signal} then it\n"
 	    "should be a list containing the Unix signal number; If\n"
-	    "@var{key} is @code{out-of-range} or @code{wrong-type-arg},\n"
+	    "@var{key} is @code{out-of-range}, @code{wrong-type-arg},\n"
+	    "or @code{keyword-argument-error}, "
             "it is a list containing the bad value; otherwise\n"
 	    "it will usually be @code{#f}.")
 #define FUNC_NAME s_scm_error_scm
@@ -100,17 +99,6 @@ SCM_DEFINE (scm_error_scm, "scm-error", 5, 0, 0,
 }
 #undef FUNC_NAME
 
-#if defined __MINGW32__ && defined HAVE_NETWORKING
-# include "win32-socket.h"
-# define SCM_I_STRERROR(err) \
-    ((err >= WSABASEERR) ? scm_i_socket_strerror (err) : strerror (err))
-# define SCM_I_ERRNO() \
-    (errno ? errno : scm_i_socket_errno ())
-#else
-# define SCM_I_STRERROR(err) strerror (err)
-# define SCM_I_ERRNO() errno
-#endif /* __MINGW32__ */
-
 /* strerror may not be thread safe, for instance in glibc (version 2.3.2) an
    error number not among the known values results in a string like "Unknown
    error 9999" formed in a static buffer, which will be overwritten by a
@@ -133,10 +121,13 @@ SCM_DEFINE (scm_strerror, "strerror", 1, 0, 0,
 #define FUNC_NAME s_scm_strerror
 {
   SCM ret;
+  int errnum = scm_to_int (err);  /* Must be done outside of the
+				     critical section below, to avoid a
+				     deadlock on errors.  */
   scm_dynwind_begin (0);
   scm_i_dynwind_pthread_mutex_lock (&scm_i_misc_mutex);
 
-  ret = scm_from_locale_string (SCM_I_STRERROR (scm_to_int (err)));
+  ret = scm_from_locale_string (strerror (errnum));
 
   scm_dynwind_end ();
   return ret;
@@ -147,7 +138,7 @@ SCM_GLOBAL_SYMBOL (scm_system_error_key, "system-error");
 void
 scm_syserror (const char *subr)
 {
-  SCM err = scm_from_int (SCM_I_ERRNO ());
+  SCM err = scm_from_int (errno);
 
   /* It could be that we're getting here because the syscall was
      interrupted by a signal.  In that case a signal handler might have
