@@ -51,6 +51,7 @@
 
 #include "libguile/validate.h"
 #include "libguile/filesys.h"
+#include "libguile/load.h"	/* for scm_i_mirror_backslashes */
 
 
 #ifdef HAVE_IO_H
@@ -770,8 +771,13 @@ SCM_DEFINE (scm_select, "select", 3, 2, 0,
 	    "This procedure has a variety of uses: waiting for the ability\n"
 	    "to provide input, accept output, or the existence of\n"
 	    "exceptional conditions on a collection of ports or file\n"
-	    "descriptors, or waiting for a timeout to occur.\n"
-	    "It also returns if interrupted by a signal.\n\n"
+	    "descriptors, or waiting for a timeout to occur.\n\n"
+
+	    "When an error occurs, of if it is interrupted by a signal, this\n"
+	    "procedure throws a @code{system-error} exception\n"
+	    "(@pxref{Conventions, @code{system-error}}).  In case of an\n"
+	    "interruption, the associated error number is @var{EINTR}.\n\n"
+
 	    "@var{reads}, @var{writes} and @var{excepts} can be lists or\n"
 	    "vectors, with each member a port or a file descriptor.\n"
 	    "The value returned is a list of three corresponding\n"
@@ -985,8 +991,8 @@ SCM_DEFINE (scm_fsync, "fsync", 1, 0, 0,
 #ifdef HAVE_SYMLINK
 SCM_DEFINE (scm_symlink, "symlink", 2, 0, 0,
             (SCM oldpath, SCM newpath),
-	    "Create a symbolic link named @var{oldpath} with the value\n"
-	    "(i.e., pointing to) @var{newpath}.  The return value is\n"
+	    "Create a symbolic link named @var{newpath} with the value\n"
+	    "(i.e., pointing to) @var{oldpath}.  The return value is\n"
 	    "unspecified.")
 #define FUNC_NAME s_scm_symlink
 {
@@ -1235,6 +1241,9 @@ SCM_DEFINE (scm_getcwd, "getcwd", 0, 0, 0,
       errno = save_errno;
       SCM_SYSERROR;
     }
+  /* On Windows, convert backslashes in current directory to forward
+     slashes.  */
+  scm_i_mirror_backslashes (wd);
   result = scm_from_locale_stringn (wd, strlen (wd));
   free (wd);
   return result;
