@@ -1,4 +1,4 @@
-/* Copyright (C) 2009, 2010, 2011, 2013, 2014 Free Software Foundation, Inc.
+/* Copyright (C) 2009, 2010, 2011, 2013-2015 Free Software Foundation, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -89,6 +89,8 @@ make_bip (SCM bv)
 
   /* Match the expectation of `binary-port?'.  */
   c_port->encoding = NULL;
+  /* XXX Manually update encoding_mode.  This will be cleaned up in 2.2.  */
+  SCM_PORT_GET_INTERNAL (port)->encoding_mode = SCM_PORT_ENCODING_MODE_ICONV;
 
   /* Prevent BV from being GC'd.  */
   SCM_SETSTREAM (port, SCM_UNPACK (bv));
@@ -123,45 +125,69 @@ bip_fill_input (SCM port)
   return result;
 }
 
+static void
+bip_end_input (SCM port, int offset)
+{
+  scm_t_port *c_port = SCM_PTAB_ENTRY (port);
+  
+  if (c_port->read_pos - c_port->read_buf < offset)
+    scm_misc_error ("bip_end_input", "negative position", SCM_EOL);
+
+  c_port->read_pos -= offset;
+}
+
 static scm_t_off
 bip_seek (SCM port, scm_t_off offset, int whence)
 #define FUNC_NAME "bip_seek"
 {
-  scm_t_off c_result = 0;
   scm_t_port *c_port = SCM_PTAB_ENTRY (port);
+  scm_t_off target;
 
-  switch (whence)
+  if (offset == 0 && whence == SEEK_CUR)
+    /* special case to avoid disturbing the putback buffer.  */
     {
-    case SEEK_CUR:
-      offset += c_port->read_pos - c_port->read_buf;
-      /* Fall through.  */
-
-    case SEEK_SET:
-      if (c_port->read_buf + offset <= c_port->read_end)
-	{
-	  c_port->read_pos = c_port->read_buf + offset;
-	  c_result = offset;
-	}
+      if (c_port->read_buf == c_port->putback_buf)
+        target = c_port->saved_read_pos - c_port->saved_read_buf
+          - (c_port->read_end - c_port->read_pos);
       else
-	scm_out_of_range (FUNC_NAME, scm_from_int (offset));
-      break;
-
-    case SEEK_END:
-      if (c_port->read_end - offset >= c_port->read_buf)
-	{
-	  c_port->read_pos = c_port->read_end - offset;
-	  c_result = c_port->read_pos - c_port->read_buf;
-	}
-      else
-	scm_out_of_range (FUNC_NAME, scm_from_int (offset));
-      break;
-
-    default:
-      scm_wrong_type_arg_msg (FUNC_NAME, 0, port,
-			      "invalid `seek' parameter");
+        target = c_port->read_pos - c_port->read_buf;
     }
+  else
+    {
+      scm_t_off base = 0;
 
-  return c_result;
+      /* If the putback buffer is currently active, this will dump its
+         contents, switch back to the main read buffer, and move
+         read_pos backwards as needed to account for the bytes that were
+         put back. */
+      if (c_port->read_buf == c_port->putback_buf)
+        scm_end_input (port);
+
+      switch (whence)
+        {
+        case SEEK_CUR:
+          base = c_port->read_pos - c_port->read_buf;
+          break;
+        case SEEK_END:
+          base = c_port->read_end - c_port->read_buf;
+          break;
+        case SEEK_SET:
+          base = 0;
+          break;
+        default:
+          scm_wrong_type_arg_msg (FUNC_NAME, 0, port,
+                                  "invalid `whence' argument");
+        }
+
+      if (offset > SCM_T_OFF_MAX - base)  /* Overflow check */
+        scm_out_of_range (FUNC_NAME, scm_from_int (offset));
+      target = base + offset;
+      if (target < 0 || target > c_port->read_end - c_port->read_buf)
+        scm_out_of_range (FUNC_NAME, scm_from_int (offset));
+
+      c_port->read_pos = c_port->read_buf + target;
+    }
+  return target;
 }
 #undef FUNC_NAME
 
@@ -174,7 +200,8 @@ initialize_bytevector_input_ports (void)
     scm_make_port_type ("r6rs-bytevector-input-port", bip_fill_input,
 			NULL);
 
-  scm_set_port_seek (bytevector_input_port_type, bip_seek);
+  scm_set_port_end_input (bytevector_input_port_type, bip_end_input);
+  scm_set_port_seek      (bytevector_input_port_type, bip_seek);
 }
 
 
@@ -307,9 +334,10 @@ cbip_setvbuf (SCM port, long read_size, long write_size)
   switch (read_size)
     {
     case 0:
-      /* Unbuffered: keep PORT's bytevector as is (it will be used in
-	 future 'scm_c_read' calls), but point to the one-byte buffer.  */
-      pt->read_buf = &pt->shortbuf;
+      /* Unbuffered: keep using PORT's bytevector as the underlying
+	 buffer (it will also be used by future 'scm_c_read' calls.)  */
+      assert (SCM_BYTEVECTOR_LENGTH (bv) >= 1);
+      pt->read_buf = (unsigned char *) SCM_BYTEVECTOR_CONTENTS (bv);
       pt->read_buf_size = 1;
       break;
 
@@ -361,6 +389,8 @@ make_cbip (SCM read_proc, SCM get_position_proc,
 
   /* Match the expectation of `binary-port?'.  */
   c_port->encoding = NULL;
+  /* XXX Manually update encoding_mode.  This will be cleaned up in 2.2.  */
+  SCM_PORT_GET_INTERNAL (port)->encoding_mode = SCM_PORT_ENCODING_MODE_ICONV;
 
   /* Attach it the method vector.  */
   SCM_SETSTREAM (port, SCM_UNPACK (method_vector));
@@ -404,9 +434,11 @@ cbip_fill_input (SCM port)
 
       if (buffered)
 	{
-	  /* Make sure the buffer isn't corrupt.  BV can be passed directly
-	     to READ_PROC.  */
-	  assert (c_port->read_buf_size == SCM_BYTEVECTOR_LENGTH (bv));
+	  /* Make sure the buffer isn't corrupt.  Its size can be 1 when
+	     someone called 'setvbuf' with _IONBF.  BV can be passed
+	     directly to READ_PROC.  */
+	  assert (c_port->read_buf_size == SCM_BYTEVECTOR_LENGTH (bv)
+		  || c_port->read_buf_size == 1);
 	  c_port->read_pos = (unsigned char *) SCM_BYTEVECTOR_CONTENTS (bv);
 	}
       else
@@ -811,7 +843,7 @@ SCM_DEFINE (scm_unget_bytevector, "unget-bytevector", 2, 2, 0,
 	}
       else
 	{
-	  if (SCM_UNLIKELY (c_start >= c_len))
+	  if (SCM_UNLIKELY (c_start > c_len))
 	    scm_out_of_range (FUNC_NAME, start);
 	  else
 	    c_count = c_len - c_start;
@@ -909,6 +941,8 @@ make_bop (void)
 
   /* Match the expectation of `binary-port?'.  */
   c_port->encoding = NULL;
+  /* XXX Manually update encoding_mode.  This will be cleaned up in 2.2.  */
+  SCM_PORT_GET_INTERNAL (port)->encoding_mode = SCM_PORT_ENCODING_MODE_ICONV;
 
   buf = (scm_t_bop_buffer *) scm_gc_malloc (sizeof (* buf), SCM_GC_BOP);
   bop_buffer_init (buf);
@@ -1068,6 +1102,8 @@ make_cbop (SCM write_proc, SCM get_position_proc,
 
   /* Match the expectation of `binary-port?'.  */
   c_port->encoding = NULL;
+  /* XXX Manually update encoding_mode.  This will be cleaned up in 2.2.  */
+  SCM_PORT_GET_INTERNAL (port)->encoding_mode = SCM_PORT_ENCODING_MODE_ICONV;
 
   /* Attach it the method vector.  */
   SCM_SETSTREAM (port, SCM_UNPACK (method_vector));
@@ -1364,12 +1400,6 @@ scm_register_r6rs_ports (void)
                             "scm_init_r6rs_ports",
 			    (scm_t_extension_init_func) scm_init_r6rs_ports,
 			    NULL);
-}
-
-void
-scm_init_r6rs_ports (void)
-{
-#include "libguile/r6rs-ports.x"
 
   initialize_bytevector_input_ports ();
   initialize_custom_binary_input_ports ();
@@ -1377,3 +1407,9 @@ scm_init_r6rs_ports (void)
   initialize_custom_binary_output_ports ();
   initialize_transcoded_ports ();
 }
+
+void
+scm_init_r6rs_ports (void)
+{
+#include "libguile/r6rs-ports.x"
+}
