@@ -61,11 +61,7 @@
 #include "libguile/stime.h"
 
 #include <unistd.h>
-
-
-#ifdef HAVE_CLOCK_GETTIME
-# include <time.h>
-#endif
+#include <time.h>				  /* Gnulib-provided */
 
 #include <sys/types.h>
 #include <string.h>
@@ -331,7 +327,7 @@ filltime (struct tm *bd_time, int zoff, const char *zname)
   return result;
 }
 
-static char tzvar[3] = "TZ";
+static const char tzvar[3] = "TZ";
 
 /* if zone is set, create a temporary environment with only a TZ
    string.  other threads or interrupt handlers shouldn't be allowed
